########################################################################################################################################################
# Written by  : Catherine J Pink
# Created on  : 28/04/2009
# Modified on : 29/04/2009 (Enhanced control script)
# Updates for publication: 12/05/2015
#
# Input files : Mouse_Jul2007_EXin.tfa
#		Rat_Nov2004_EXin.tfa
#		HMD_Rat5.rpt
#		mouse.rna.gbff
#		rat.rna.gbff
#		mmu_rno_introns_Y.tab
#		Zfy_int_aln_ed.tfa
#		NM_011667.txt
#		mm9ToMm8.over.chain
#		RD_46CESCave_Sm300_081128.txt
#		RD_D3ESCave_Sm300_081128.txt
#		RD_iPSave_Sm300_081128.txt
#		RD_TT2ESCave_Sm300_081128.txt
#
# Ensure that lagan.pl and liftOver are in the path - see readme.txt file
#
#
#
#
########################################################################################################################################################
puts "\n\n***********************************************************"
puts "************ STARTING MOUSE REPLICATION PROJECT ***********"
puts "***********************************************************\n"

# CREATE SCRIPT LIBRARY AND ENSURE ALL REQUIRED SCRIPTS ARE PRESENT
puts "Creating script library"
# Check the script library exists and if not, exit the program
if {![file exists mm_rn_Ki_RT_with_filter_library]} {
	puts "ERROR...The mm_rn_Ki_RT_with_filter script library can not be located in [pwd]"
	puts "Exiting the program"
	exit
}
# Check that the script library contains the correct scripts
# Move to the folder containing the script library
cd mm_rn_Ki_RT_with_filter_library
# Set a list of the script files that should be present as a variable
set scriptlist {
	openfile.tcl
	Knc.pl
	calculateMean.tcl
	calculateMedian.tcl
	INTRONS01_extractgenes.tcl
	INTRONS02_extractintrons.tcl
	INTRONS03_orthologlist.tcl
	INTRONS04_convertgeneid.tcl
	INTRONS05_identifyorthologs.tcl
	INTRONS06_extractorthologs.tcl
	INTRONS07_ratYintrons.tcl
	INTRONS08_alignintrons.tcl
	INTRONS09_alignmentparameters.tcl
	INTRONS14_concatenategenes.tcl
	INTRONS15_concatenatechromosomes.tcl
	INTRONS16_Kintron.tcl
	INTRONS17_Kgene.tcl
	INTRONS18_Kchromosome.tcl
    19_liftoverGENES.tcl
	20_liftoverINTRONS.tcl
	21_combine_replication_tables.tcl
	22_allocate_RepTime_Kgenes_genic.tcl
	23_allocate_RepTime_Kintrons_intronic.tcl
	24_allocate_RepTime_Kintrons_genic.tcl
	25_GT_skew.tcl
	26_rearrangement_index.tcl	
	
}
# Go through the list of files that should be present in the script library
foreach script $scriptlist {
	# If the script is not present in the script library, exit the program
	if {![file exists $script]} {
		puts "ERROR...Missing script : $script"
		puts "Exiting the program"
		exit
	}
}
# Print to screen information that all correct scripts are present in the script library directory
puts "All scripts present in tcl script directory"
# Identify the full directory name of the script library
set current_directory [exec pwd]
# Create the index required to use the script library
auto_mkindex $current_directory *.tcl
lappend auto_path $current_directory
# Print to screen information that the script library was created
puts "Tcl script library created"
# Go back to the original folder
cd ..
# Unset variables
unset script scriptlist current_directory


# ENSURE ALL PRIMARY DATA INPUT FILES ARE PRESENT
puts "Checking input data files present"
set datainputlist {
	Mouse_Jul2007_EXin.tfa
	Rat_Nov2004_EXin.tfa
	HMD_Rat5.rpt
	mouse.rna.gbff
	rat.rna.gbff
	mmu_rno_introns_Y.tab
	Zfy_int_aln_ed.tfa
	mm9ToMm8.over.chain
	RD_46CESCave_Sm300_081128.txt
	RD_D3ESCave_Sm300_081128.txt
	RD_iPSave_Sm300_081128.txt
	RD_TT2ESCave_Sm300_081128.txt
}
# Go through the list of input files that should be present
cd Input_Files
foreach datafile $datainputlist {
    # If the script is not present in the script library, exit the program
    if {![file exists $datafile]} {
	puts "ERROR...Missing data file : $datafile"
	puts "Exiting the program"
	exit
   }
}
cd ..
# Print to screen information that all correct scripts are present in the script library directory
puts "All input data files present"
# Unset variables
unset datainputlist datafile


# If none exists, create output directory
if {![file exists Output_Files]} {
	exec mkdir Output_Files
}
if {![file exists Intron_Data]} {
	exec mkdir Intron_Data
}

# Run procedures in turn
puts "\n-----------------------------------------------------------"
puts "         RUNNING PROCEDURE SCRIPTS FOR ANALYSIS NOW        "
puts "-----------------------------------------------------------"

# ***************************************************
# GENERATE NEW SUBSTITUTION RATES INCLUDING BY INTRON
# ***************************************************
INTRONS01_extractgenes
INTRONS02_extractintrons
INTRONS03_orthologlist
INTRONS04_convertgeneid
INTRONS05_identifyorthologs
INTRONS06_extractorthologs
INTRONS07_ratYintrons
INTRONS08_alignintrons
INTRONS09_alignmentparameters
INTRONS14_concatenategenes
INTRONS15_concatenatechromosomes
INTRONS16_Kintron
INTRONS17_Kgene
INTRONS18_Kchromosome

# ***************************************************
# CONVERT POSITIONS FROM BUILD MM9 TO MM8
# ***************************************************
19_liftoverGENES
20_liftoverINTRONS

# ***************************************************
# ASSIGN REPLICATION TIMES TO SUBSTITUTION RATES
# 3 POSSIBLE METHODS:
#	1. GENIC REPLICATION TIMES TO GENIC SUBSTITUTION RATES 			(Yn = 3)		allocate_RepTime_Kgenes_genic
#	2. INTRONIC REPLICATION TIMES TO INTRONIC SUBSTITUTION RATES	(Yn = 0)		allocate_RepTime_Kintrons_intronic
#	3. GENIC REPLICATION TIMES TO INTRONIC SUBSTITUTION RATES		(Yn = 20)		allocate_RepTime_Kintrons_genic
# ***************************************************

21_combine_replication_tables
22_allocate_RepTime_Kgenes_genic
23_allocate_RepTime_Kintrons_intronic
24_allocate_RepTime_Kintrons_genic

# ***************************************************
# OTHER BITS NEEDED FOR THE ANALYSIS
# ***************************************************
# Calculate GT bias in each orthologous gene
25_GT_skew
26_rearrangement_index
# ***************************************************

puts "\n-----------------------------------------------------------"
puts "Variables open in mm_rn_Ki_RT_with_filter : [info local]"
puts "-----------------------------------------------------------"


puts "\n***********************************************************"
puts "************ FINISHED MOUSE REPLICATION PROJECT ***********"
puts "***********************************************************\n\n"
exit
