########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 15/10/2007 (As part of MMBIntrons work)
# Modified on 	: 08/05/2009 (Change of directory names and script structure to fit in with current analysis)
#							 (Removal of counts to be output to information file)
# 						     (Deletion of unwanted ELIMINATED gene files as checking done as part of MMB analysis and to reduce file number)
#				  28/05/2009 (Movement of Y-linked (not Y_random) NM_011667.txt file from Input_Files to GENES_Mouse directory)
#
# Input files 	: Mouse_Jul2007_EXin.tfa (Both downloaded from UCSC table browser)
#				  Rat_Nov2004_EXin.tfa
#				  NM_011667.txt (Manually obtained from UCSC: Y-lined position information)					
# 
# Output files  : NM_*****.txt for each gene stored in directories GENES_Organism
#
# Procedure to extract regions from USCS download file and store regions from each gene in a separate file identified by RefSec ID
#
# Process of gene extraction from the mouse and rat input refseq files in a FASTA format (one region per FASTA, EXONS in uppercase,introns in lower case
# Create directories to store accepted (GENES_$organism) and eliminated (ELIMINATED_$organism) gene files for each organism
# Due to the large file size which tcl cannot open in one go, the input FASTA file is instead read in a line at a time until there is no longer a line to read in
# The structure of the program is shown in the flow diagram
# Sequences were concatenated per region and sent in a FASTA format to the output file, one GENE per file
# Some refseq IDs appear multiple genes, probably due to reannotation of duplicate genes. Often the duplicate gene consisted of a single exon
# Duplicate genes or single exon genes were eliminated from the analysis and saved in the appropriate ELIMINATED directory (removed at end of script)
# Save non-duplicate genes containing multiple exons and introns in the relevant GENES directory as appropriate for the organism
#
#
#	 |--------->If line exists - read in line-----------------<--------------------------------<------------------------------------|
#	 |				 |														  														|
#    |         Does line start > (annotation line) ? -----no----------- Line contains sequence										|
#    |               |                                                  Append sequence string to tempregion-------->---------------|
#    |              yes																												|
#    |               |																												|
# 	 |   	This is an annotation line																								|
#	 |				 |																												|
#    |       Is this first gene of genome? -------------no-----------Send previous gene regions and annotations to output file		|
#    |               |																	|											|
#    |              yes																	|											|
#    |               | <----------------------------------------------------------------|											|
#	 |				 |																												|
#	 |		 Identify refseq ID (and region (intron/exon) number if multiple region gene) 											^
#	 ^				 |																												|
#	 |		Are there multiple regions?-------------------no-----------set region number as 0										|
#	 |				 |															|													|
#	 |				yes															|													|
#	 |				 |	<-------------------------------------------------------|													|
#	 | 				 |																												|
#	 | 		Is region number 0? ---------------no----------------------------This the start of the next region for the gene			|
#	 |				 |                                                                  |											|
#	 |				yes													     Send previous region to output file for this gene-->---|
#	 |				 |																	
#	 |		Is this the first region in the genome?-----no--------Send previous region to output file
#    | 				 |														 |
#    | 				yes													Close output file 													
#    | 				 |														 |
#    |--<--Create output file for new gene		   				Move to GENES output directory
#	 				 ^														|
#	 |------>--------|														|
#	 |				 														v
#	 |				 														|
#	 |			Move back to main directory-----------no---------Does filename already exist?
#	 |				 |														|
#	 |		Move output file to GENES output directory			 		   yes
#	 |				 |								     					|	
#	 ^				 |								Generate novel filename for file that already exists (incr count in filename until unique)
#	 |				 |														|
#	 |				 |		   								Move file to ELIMINATED output directory
#	 |				 v														|
#	 |				 |														|
# 	 |				 |														v
#	 |			     |														|
#    |--------<---------------------------------<---------------------------|
#
########################################################################################################################################################

package provide INTRONS01_extractgenes 1.0

proc INTRONS01_extractgenes {} {
	
	puts "-----------------------------------------------------------"
	puts "01             EXTRACT GENES FROM UCSC FILES               \n"
	    
    # Make variables global
    global contents
    
    # Find UCSC genome files in Input_Files directory
	cd Input_Files
	set refseqfiles [glob *EXin.tfa]
    cd ..

	# Extract genes from UCSC genome file for each organism in turn
	foreach refseqfile $refseqfiles {
	
		# Determine organism
		regexp {([A-Za-z]+)_.+} $refseqfile all organism
		puts "Extracting $organism genes"
		
		cd Intron_Data
		# Create a new directory to store each gene file
		if {![file exists GENES_$organism]} {
	    	puts "No GENES_$organism directory was found"
	    	exec mkdir GENES_$organism
	    	puts "New GENES_$organism directory was created"
		} elseif {[file exists GENES_$organism]} {
	    	exec rm -r GENES_$organism
	    	puts "GENES_$organism directory was found and removed"
	    	exec mkdir GENES_$organism
	    	puts "New GENES_$organism directory was created"
		}
		# Create a new directory to store eliminated gene files in
		if {![file exists ELIMINATED_$organism]} {
	    	puts "No ELIMINATED_$organism directory was found"
	    	exec mkdir ELIMINATED_$organism
	    	puts "New ELIMINATED_$organism directory was created"
		} elseif {[file exists ELIMINATED_$organism]} {
	    	exec rm -r ELIMINATED_$organism
	    	puts "ELIMINATED_$organism directory was found and removed"
	    	exec mkdir ELIMINATED_$organism
	    	puts "New ELIMINATED_$organism directory was created"
		}
		cd ..
	
		# Open RefSeq file in the Input_Files directory
		cd Input_Files
		set in [open $refseqfile r]
		cd ..
		puts "Reading in $refseqfile file"
	
		# Initiate counts for program information data
		set regioncount 0
			
		# Initiate lists for checking program works accurately
		set allgeneIDs {}
		set singleregiongenes {}
		set multipleregiongenes {}
		set filealreadyexists {}
		
		
		# Move to the Intron_data, and read in file line by line.
		cd Intron_Data
		
		# If the line contains information, parse each line as it's read
		while {[gets $in line] != -1} {
	    	
	    	# If the line contains > is start of region 
	    	if {[regexp {>} $line all]} {
				incr regioncount

				# If not the first gene of the genome, set the previous refseqID as different variable prior to re-setting refseqID for a the new gene
				if {$regioncount > 1} {
		    		set filetomove $refseqID
		    		# Unset variables so they can't be reused
		    		unset refseqID regionnumber
				}

				# Determine the region details
				if {[regexp {>.+_refGene_(\w+[_]*[0-9]+)_([0-9]+)} $line all]} {
		    		# Gene contains multiple regions
		    		regexp {>.+_refGene_(\w+[_]*[0-9]+)_([0-9]+)} $line all refseqID regionnumber
		    		# Append gene ID to all gene ID list
		    		lappend allgeneIDs $refseqID	
		    		lappend multipleregiongenes $refseqID
				} else {
		    		# Gene contains only a single region
		    		regexp {>.+_refGene_(\w+[_]*[0-9]+)} $line all refseqID
		    		set regionnumber 0
		    		lappend allgeneIDs $refseqID
					lappend singleregiongenes $refseqID
				}
			
				# If the first region of a gene, deal with output files to store information
				if {$regionnumber == 0} {
		    
		    		# If not the first gene/region, send final region of previous gene to output file, close file and move to organism gene directory
		    		if {$regioncount > 1} {
						puts $out "$tempregion"
						close $out
						unset out
			
						# Determine if a file exists for a duplicate refseqID
						cd GENES_$organism
						# If file already exists, send duplicate to eliminated directory and store duplicate refseqID		
						if {[file exists $filetomove.txt]} {
			    			cd ..
			    
			    			# Ensure each filename is unique so duplicate files aren't overwritten and can be checked
			    			set filenamecount 0
							cd ELIMINATED_$organism
			    			# Create a unique filename for duplicate refseqID gene files
			    			set filename $filetomove-$filenamecount.txt
			    			while {[file exists $filename]} {
								incr filenamecount
								set filename $filetomove-$filenamecount.txt
			    			}
							cd ..
			    			# Move duplicate refseqID gene files to eliminated directory with unqiue filename			    
			    			exec mv $filetomove.txt ELIMINATED_$organism/$filetomove-$filenamecount.txt
			    			# Unset variable so it can't be reused
			    			unset filenamecount
			    			# Increase counts of duplicate filenames (refseqIDs)
			    			lappend filealreadyexists $filetomove

							# If file does not already exist, send new file to genes directory
						} else {
			    			cd ..
			   				exec mv $filetomove.txt GENES_$organism/$filetomove.txt
						}
						# Unset variables
						unset filetomove		
		    		}
 
		    		# Create output file for the gene to store all regions in 
		    		set out [open $refseqID.txt w]    
		    
					# If not the first region send the previous region to the output file
				} elseif {$regionnumber > 0} {
		    		puts $out "$tempregion"
		    		unset tempregion
				}
		
				# Initiate a variable to store all information for the current region
				set tempregion "$line\n"
		
				# If line does not start > append nucleotide sequence to current region variable	
	    	} else {
				append tempregion $line
	    	}		
		}	
	
		# Send final region in current gene to output file, close file and move to organism gene directory
		puts $out "$tempregion"
		close $out
		
		exec mv $filetomove.txt GENES_$organism/$filetomove.txt
		unset out filetomove tempregion refseqID regionnumber
		
		# Close input file
		close $in
	
		# Counts to ensure script is correctly extracting all genes and gene regions
		set uniqueallgeneIDs [lsort -unique $allgeneIDs]
		set uniquesingleregiongenes [lsort -unique $singleregiongenes]
		set uniquemultipleregiongenes [lsort -unique $multipleregiongenes]
		set uniquefilealreadyexists [lsort -unique $filealreadyexists]
	
	
		# Determine how many UCSC refseqIDs started XM_ (provisional genes) for comparison with MGI orthologue database
		set countXrefseqIDs 0
		foreach geneID $uniqueallgeneIDs {
	    	if {[regexp {X} $geneID all]} {
				incr countXrefseqIDs
	    	}
		}
		unset geneID	
	
		# File count before removal of duplicate refseqID files
		cd GENES_$organism
		set filelistbefore [glob *.txt]
		cd ..
		set FilesBeforeDuplicateRemoval [llength $filelistbefore]
		unset filelistbefore
	
		# Move duplicate refseqID file to eliminated directory
		cd GENES_$organism
		foreach fl $uniquefilealreadyexists {    
	    	# Ensure each filename is unique so duplicate files aren't overwritten and can be checked
	    	cd .. 
	    	cd ELIMINATED_$organism
	    	# Create a unique filename for duplicate refseqID gene files
	    	set filenamecount 0
	    	set filename $fl-$filenamecount.txt
	    	while {[file exists $filename]} {
				incr filenamecount
				set filename $fl-$filenamecount.txt
	    	}
	    	cd ..
	    	cd GENES_$organism
	    	# Move duplicate refseqID gene files to eliminated directory with unqiue filename			    
	    	exec mv $fl.txt ../ELIMINATED_$organism/$fl-$filenamecount.txt
	    	unset filenamecount filename
		}
		set filelistafter [glob *.txt]
		cd ..
		set FilesAfterDuplicateRemoval [llength $filelistafter]
		unset fl filelistafter
	
		# Determine whether refseqIDs corresponding to single region genes have also been assigned to multiple region genes
		set multiple $uniquemultipleregiongenes
		set singleandmultiple {}
		set singleonly {}
		foreach s $uniquesingleregiongenes {
	    	# Determine if single region refseqID is also a multiple region refseqID
	    	set pos [lsearch $multiple $s]
	    	if {$pos >= 0} {
				lappend singleandmultiple $s
				# Remove refseqID from search list to increase spead of subsequent searches
				set multiple [lreplace $multiple $pos $pos]
	    	} else {
				lappend singleonly $s
	    	}
	    	unset pos
		}
		unset s multiple
	
		# Return to main directory once all gene files output to directory in Intron_Data directory
		cd .. 
		
		
		# Unset variables
		unset line organism in regioncount allgeneIDs uniqueallgeneIDs 
		unset singleregiongenes uniquesingleregiongenes multipleregiongenes uniquemultipleregiongenes 
		unset filealreadyexists uniquefilealreadyexists singleonly singleandmultiple countXrefseqIDs 
		unset FilesBeforeDuplicateRemoval FilesAfterDuplicateRemoval  
    }
    # Unset variables
    unset refseqfile refseqfiles all
       
	# Remove directories of eliminated genes
	cd Intron_Data
	exec rm -r ELIMINATED_Mouse
	exec rm -r ELIMINATED_Rat
	cd ..

	# Move manually downloaded file NM_011667.txt to GENES_Mouse directory
	cd Input_Files
	exec cp NM_011667.txt ../Intron_Data/GENES_Mouse/NM_011667.txt
	cd ..
	
    puts "\nVariables open in INTRONS01_extractgenes : [info local]"
    
return	
}