########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 26/11/2007 (As part of MMBIntrons work)
# Modified on 	: 12/05/2009 (Change of directory names and script structure to fit in with current analysis)
#				  12/05/2009 (Removal of output statistics and trimming out details printed to screen)
#				  
# Input files 	: rn-$ratRefSeq-$Rintronnumber.txt (Orthologous mouse introns stored in Orth-$orthologcount-chr-$chromosome directory, in ORTHOLOGS directory)
# 				  mm-$mouseRefSeq-$Mintronnumber.txt (Orthologous rat introns stored in Orth-$orthologcount-chr-$chromosome directory, in ORTHOLOGS directory)
#				  One pair of orthologous files per intron		  
#				  
# Output files  : $orthcount-$intronnumber-$chromosome.nuc (For each aligned intron)
#				  Stored within an new ALIGNMENTS directory
#				  Stored within Intron_Data directory
# 				  
# Within ORTHOLOGS directory containing directories of all orthologous genes
# Go into each ortholog directory containing orthologous introns
# Determine the number of files within the ortholog directory and therefore the number of introns for that orthologs gene
# For each intron, identify the files containing orthologous introns
# Align the pair of introns and save alignment under filename orthcount-intronnumber-chromosome.nuc
# Move the file containing the intron alignment to ALIGNMENTS directory 
#
########################################################################################################################################################

package provide INTRONS08_alignintrons 1.0

proc INTRONS08_alignintrons {} {
	
	puts "-----------------------------------------------------------"
	puts "08                    ALIGNING INTRONS                     \n"
	
    # Make variables global
    global contents

	# Move to Intron_Data directory and do all work in this directory
	cd Intron_Data

    
    # Create directory to store all alignments in
    if {![file exists ALIGNMENTS]} {
		puts "No ALIGNMENTS directory was found"
		exec mkdir ALIGNMENTS
		puts "New ALIGNMENTS directory created"
    } elseif {[file exists ALIGNMENTS]} {
		exec rm -r ALIGNMENTS
		puts "ALIGNMENTS directory was found and removed"
		exec mkdir ALIGNMENTS
		puts "New ALIGNMENTS directory created"
    }

	# Create file to store failed alignment names in
	set outTXT [open Failed_alignments.txt w]
    puts $outTXT  "Failed_Alignment_Count\tOrtholog\tIntron_Number"

    # Move into ORTHOLOGS directory and create list of directories to work through
    cd ORTHOLOGS
    set OrthologDirectories [lsort [glob Orth*]]

    # Initiate a counts
    set countGenes 0
    set countIntrons 0
    
    # Initiate counts for program checking
    set countFailedAlignments 0
    set countAlignmentFilesMissing 0
    
    # Work through list of ortholog directories
    foreach OrthDirectory $OrthologDirectories {
		incr countGenes
	
		# Extract chromosome and ortholog count details from directory name for alignment file naming
		regexp {Orth-(.+)-chr-(.+)} $OrthDirectory all orthcount chromosome
		
		if {[expr {$countGenes %200}] == 0} {puts "Aligning $countGenes of [llength $OrthologDirectories] orthologous genes (not introns)"}

		# Move into ortholog directory
		cd $OrthDirectory
		
		# Generate a list of mouse intron numbers for this ortholog
		set IntronNumbersMOUSE {}
		set intronfilesMOUSE [glob mm*-NM_*.txt]
		foreach intronfile $intronfilesMOUSE {
			regexp {mm-NM_[\w]+-([0-9]+).txt} $intronfile all intronMOUSE
			lappend IntronNumbersMOUSE $intronMOUSE
			unset intronMOUSE all
		}
		unset intronfile intronfilesMOUSE
		set IntronNumbersMOUSE [lsort -integer $IntronNumbersMOUSE]
		
		# Generate a list of rat intron numbers for this ortholog
		set IntronNumbersRAT {}
		set intronfilesRAT [glob rn*-NM_*.txt]
		foreach intronfile $intronfilesRAT {
			regexp {rn-NM_[\w]+-([0-9]+).txt} $intronfile all intronRAT
			lappend IntronNumbersRAT $intronRAT
			unset intronRAT all
		}
		unset intronfile intronfilesRAT
		set IntronNumbersRAT [lsort -integer $IntronNumbersRAT]
				
		# Ensure that the same set of intron numbers have been identified for each species. If not, exit the program as there is a scripting error
		if {$IntronNumbersMOUSE != $IntronNumbersRAT} {
			puts "Intron lists not the same : Mouse : $IntronNumbersMOUSE    Rat : $IntronNumbersRAT"
			exit
		} 
		
		# Align each pair of introns determined from list of intron numbers for this chromosome (therefore accounting for incomplete Y-linked intron data set)
		foreach intronnumber $IntronNumbersMOUSE {
	    	# Increase count of introns dealt with
	    	incr countIntrons
			
	    	# Identify orthologous intron files
	    	set intronfiles [lsort [glob *-NM_*-$intronnumber.txt]]
	    	set file1 [lindex $intronfiles 0]
	    	set file2 [lindex $intronfiles 1]

	   		# Generate new filename for reformatted output file
	   		set alignmentfilename "Orth-$orthcount-Intron-$intronnumber-Chr-$chromosome.nuc"
	    
	   		# Align orthologous intron files
	   		if {[catch {exec lagan.pl $file1 $file2 -mfa -fastreject -out $alignmentfilename 2> /dev/null} errormsg]} {
				incr countFailedAlignments
				puts "** Alignment failed : Ortholog $orthcount , Intron $intronnumber"
				puts $outTXT "$countFailedAlignments\t$orthcount\t$intronnumber"
	   		}
	   
	   		# If alignment successful, move alignment file to ALIGNMENTS directory
	   		if {[file exists $alignmentfilename]} {
				# Move alignment file to ALIGNMENTS directory
				exec mv $alignmentfilename ../../ALIGNMENTS/$alignmentfilename
			# If alignment is unsuccessful, increase counts of missing files and total errors
	   		} else {
				incr countAlignmentFilesMissing
	    	}
	
	    	# Unset variables
	    	unset alignmentfilename intronfiles file1 file2
		}		
		# Unset variables
		unset intronnumber IntronNumbersMOUSE IntronNumbersRAT orthcount chromosome
		# Move back to main orthologs directory
		cd ..
	}
    # Unset variables
    unset OrthDirectory OrthologDirectories
    # Move back to main INTRONS directory
    cd ..
    
    # Determine number of aligned intron files created
    cd ALIGNMENTS
    set NumberIntronFiles [llength [glob *.nuc]]
    cd ..
    
    # Summary Information
    puts "\nNumber of genes aligned               : $countGenes"
    puts "Number of intron alignment attempted  : $countIntrons"
    puts "Number of introns not aligned         : $countFailedAlignments"
    puts "Number of alignment files missing     : $countAlignmentFilesMissing"
    puts "Number of intron files created        : $NumberIntronFiles"
       
    # Unset variables
    unset errormsg countGenes countFailedAlignments countIntrons countAlignmentFilesMissing NumberIntronFiles
    
	# Close output file containing failed alignments
	close $outTXT
	unset outTXT

	# Return from Intron_Data directory to main directory
   	cd ..

    puts "\nVariables open in INTRONS08_alignintrons : [info local]"

return
}
