########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 28/11/2007 (As part of MMBIntrons work)
# Modified on 	: 12/05/2009 (Change of directory names and script structure to fit in with current analysis)
#				  12/05/2009 (Removal of output statistics and trimming out details printed to screen)
#				  12/05/2009 (Removal of file output to ALIGNMENTS_failed to reduce file numbers)
#				  12/05/2009 (Modification to remove alignment parameter tests to separate procedures)
#				  13/05/2009 (Parameters tests and runs test moved to separate procedures)
#				  13/05/2009 (Indels removed prior to runs test to make runs test simpler)
#				  13/05/2009 (OK alignments output to file and moved to ALIGNMENTS_final directory)
#				  13/05/2009 (Addition of code to output details of each intron to a single file)
#				  14/05/2009 (Removal of first introns - Under selective constraints)
#				  09/06/2009 (Output of files failing the runs test to a new ALIGNMENTS_failed directory)
#
# Modificaion aims :
# 			Test alignments against alignment parameters
#			Test alignments against the runs test
#			For alignments that pass these tests:
#				Separate out first introns
#				Remove indels from non-first introns
#				Output all cleared introns to a new folder ready for substitutuion rate testing
#				Output details of ALL intron results to a single file for reference and counting (ie. pass or fail by 0 or 1 for column sums)
# 			
#
# Input files 	: All Orth-$ortholog-Intron-$intron-Chr-$chromosome.nuc files in ALIGNMENTS directory
# 				  (Files output by tcl script INTRONS08_alignintrons.tcl)
#				  		  
# Output files  : Alignment_Checking_Details.txt (Contains details about tests performed on each alignment)
# 				  Files in ALIGNMENTS_final directory
#				  These files contain indel free alignments with 30bp trimmed from each end that have passed both alignment parameters and the runs test.
#				  These files constitute the final intronic dataset ready for calculation of substitution rates.
#				  All files are in the correct phylip format accepted by the perl script Knc.pl
#				  
# 				  
#
#
#
#
# ALIGNMENT PARAMETERS *BEFORE* REMOVAL OF 30bp FROM EACH END
# RUNS TEST *AFTER* REMOVAL OF 30 bp FROM EACH END
#
# Create directories to store final (ALIGNMENTS_final) alignments in
# Move to ALIGNMENTS directory and go through each intron alignment file
# Extract aligned sequences and determine alignment characteristics (length, number of aligned nucleotides, number of indels)
# Determine whether alignments passes LDH alignment parameters:
# Alignment parameter 1 : Alignment length / Longest sequence < 1.16 (if > 1.16 = failed)
# Alignment parameter 2 : Number of indels / Number of aligned bases < 0.084 (if > 0.084 = failed)
# For intron alignments that pass both parameters & > 61 characters, trim 30 characters from each end to control for splice site conservation
# Remove all indels from the alignment leaving only aligned nucleotides.
# On sequences > 1 character, perform runs test to ensure distribution of switches between diverged and conserved bases is normal (alternative for sequences = 1 is to pass runs test as no switches possible)
# For every intron alignments, output results to a new file (Checked or failed ie failed60BP, failedRT, failed1AP or failed2AP)_orthcount-introncount-chromosome.nuc
# Files containing intron alignments that passed both parameters and the runs test moved to ALIGNMENTS_final
# Procedure to check alignment parameters based on LDH manual alignment
#
########################################################################################################################################################

package provide INTRONS09_alignmentparameters 1.0

proc INTRONS09_alignmentparameters {} {

	puts "-----------------------------------------------------------"
	puts "09   CHECKING ALIGNMENT AGAINST PARAMETERS & RUNS TEST     \n"
		
	# Make variables global
	global contents
	global parameter1
	global parameter2
	global alignmentNOINDELSmouse
	global alignmentNOINDELSrat
	global runstest
	
	global ratio1
	global ratio2
	
	global COUNTalignednucleotides
	global COUNTconserved
	global COUNTdiverged
	global COUNTswitches
	global percentagedivergence
	global Expected
	global Observed
	global numberindels1
	global numberindels2
	global numberindels	
 	
    # Move to Intron_Data directory and do all work in this directory
	cd Intron_Data
		
	# Create directory to store all final alignments in
	if {![file exists ALIGNMENTS_final]} {
		puts "No ALIGNMENTS_final directory was found"
		exec mkdir ALIGNMENTS_final
		puts "New ALIGNMENTS_final directory created\n"
	} elseif {[file exists ALIGNMENTS_final]} {
		exec rm -r ALIGNMENTS_final
		puts "ALIGNMENTS_final directory was found and removed"
		exec mkdir ALIGNMENTS_final
		puts "New ALIGNMENTS_final directory created\n"
	}
	
	if {![file exists ALIGNMENTS_failedRT]} {
		puts "No ALIGNMENTS_failedRT directory was found"
		exec mkdir ALIGNMENTS_failedRT
		puts "New ALIGNMENTS_failedRT directory created\n"
	} elseif {[file exists ALIGNMENTS_failedRT]} {
		exec rm -r ALIGNMENTS_failedRT
		puts "ALIGNMENTS_failedRT directory was found and removed"
		exec mkdir ALIGNMENTS_failedRT
		puts "New ALIGNMENTS_failedRT directory created\n"
	}
   
 
	# Create output file to store details of introns failing the runs test
	set outDetails [open Alignment_Checking_Details.xls w]
	puts $outDetails "OrthID\tIntron\tMouse_Chr\tFinalDataset\tPARAMETER_1_Ratio(1.16)\tParameter1\tPARAMETER_2_Ratio(0.084)\tParameter2\tRUNSTEST_Expected_Switches/Base\tRUNSTEST_Observed_Switches/Base\tRunsTest\tAlignment_Length_Original\t_Alignment_Length_TRIMMED\tAlignment_Length_NOINDELS\tSequence_Length_1\tSequence_Length_2\tNumber_Indels_1\tNumber_Indels_2\tNumber_Indels_Total\tRUNSTEST_Number_alignednucleotides\tRUNSTEST_Number_conserved\tRUNSTEST_Number_diverged\tRUNSTEST_Number_switches\tRUNSTEST_Percentage_divergence"


	# Obtain list of all alignment files
	cd ALIGNMENTS
	set alignmentfilelist [glob *.nuc]

set numberintrons [llength $alignmentfilelist]
set introncount 0

	# Check the parameters of each alignment in turn
	foreach alignmentfile $alignmentfilelist {
		
		# Identify ortholog details to use as additional information for orthologous introns that fail the runs test
		regexp {Orth-([0-9]+)-Intron-([0-9]+)-Chr-([\w]+)\.nuc} $alignmentfile all ortholognumber intronnumber chromosome

incr introncount
puts "$introncount of $numberintrons == $alignmentfile "
	
		# Open alignment file
		openfile $alignmentfile
	
		# Separate two introns from file contents
		set contents [split $contents =]
		set introns [split [lindex $contents 0] >]
		set intron1 [lindex $introns 1]
		set intron2 [lindex $introns 2] 
		unset contents introns

		# Extract sequence and alignment from mouse intron
		regsub {\n} $intron1  intron1
		set intron1 [split $intron1 ]
		set annotation1 [lindex $intron1 0]
		set alignment1 [lindex $intron1 1]
		regsub -all {\s} $alignment1 "" alignment1
		regsub -all {\-} $alignment1 "" sequence1
		unset intron1



		# Extract sequence and alignment from rat intron
		regsub {\n} $intron2  intron2
		set intron2 [split $intron2 ]
		set annotation2 [lindex $intron2 0]
		set alignment2 [lindex $intron2 1]
		regsub -all {\s} $alignment2 "" alignment2
		regsub -all {\-} $alignment2 "" sequence2
		unset intron2
	
		set alignmentlengthORIGINAL [string length $alignment1]

		# Check to ensure alignments correctly extracted and are of the same length
		if {[string length $alignment1] != [string length $alignment2]} {
			puts "ERROR...$alignmentfile alignments not the same length"
			exit
		}

		# *****************************************************************************
		# ALIGNMENT PARAMETER 1 : Alignment length / Longest sequence < 1.16 (if > 1.16 = failed)
		
		INTRONS10_parameter1 $alignment1 $alignment2
		
		# ALIGNMENT PARAMETER 2 : Number of indels / Number of aligned bases < 0.084 (if > 0.084 = failed)
		
		INTRONS11_parameter2 $alignment1 $alignment2
	
		# *****************************************************************************

			
		# Both parameters passed and alignment > 61 positions (So long enough for runs test)
		if {$parameter1 == "passed" && $parameter2 == "passed" && [string length $alignment1] > 61} {

			# Remove 30 characters from each end of alignment to control for selection on splice sites
			set trimmedalignment1 [string range $alignment1 30 [expr {[string length $alignment1] - 31}] ]
			set trimmedalignment2 [string range $alignment2 30 [expr {[string length $alignment2] - 31}] ]
	
			set alignmentlengthTRIMMED [string length $trimmedalignment1]
	
			# Check trimmed alignment lengths equal.
			if {[string length $trimmedalignment1] != [string length $trimmedalignment2]} {exit}
		
		
			# *****************************************************************************
			# REMOVE INDELS NOW RATHER THAN LATER
	
			INTRONS12_removeindels $trimmedalignment1 $trimmedalignment2

			set noindelalignment1 $alignmentNOINDELSmouse
			set noindelalignment2 $alignmentNOINDELSrat
			
			set alignmentlengthNOINDELS [string length $noindelalignment1]
			
			# *****************************************************************************
	
			# *****************************************************************************
			# RUNS TEST!!!!!!

			INTRONS13_runstest $noindelalignment1 $noindelalignment2	    		    
			
			# *************************************************************************
	
	
			# Output alignments that passed the runs test to file and move to ALIGNMENTS_final directory
			# Do not output first introns
			if {$runstest == "passed" && $intronnumber != 1} {
				
				set finaldataset Included
				
				set out [open passed_$alignmentfile w]
				puts $out "\t2\t[string length $noindelalignment1]\n>$annotation1\n$noindelalignment1\n>$annotation2\n$noindelalignment2"

				close $out
				unset out
				
				# Move new file to ALIGNMENTS_final directory
				exec mv passed_$alignmentfile ../ALIGNMENTS_final/passed_$alignmentfile
				
			} else {
				set finaldataset Purged
			}
			
			
			# Move alignments that failed the runs test and are not first introns to ALIGNMENTS_failedRT directory
			if {$runstest == "failed" && $intronnumber != 1} {
				set out [open failed_$alignmentfile w]
				puts $out "\t2\t[string length $noindelalignment1]\n>$annotation1\n$noindelalignment1\n>$annotation2\n$noindelalignment2"
				close $out
				unset out
				exec mv failed_$alignmentfile ../ALIGNMENTS_failedRT/failed_$alignmentfile
			}
			
	
			# Unset variables
			unset trimmedalignment1 trimmedalignment2
			unset noindelalignment1 noindelalignment2
			unset alignmentNOINDELSmouse alignmentNOINDELSrat
		
		# Set output details for introns that failed the alignment parameters and so were not subjected to the runs test.
		} else {
			set runstest NA
			set alignmentlengthTRIMMED NA
			set alignmentlengthNOINDELS NA
			set COUNTalignednucleotides NA
			set COUNTconserved NA
			set COUNTdiverged NA
			set COUNTswitches NA
			set percentagedivergence NA
			set Expected NA
			set Observed NA
			set finaldataset Purged
		}
	
		
		# SEND INTRON DETAILS TO OUTPUT DETAILS FILE
		puts $outDetails "$ortholognumber\t$intronnumber\t$chromosome\t$finaldataset\t$ratio1\t$parameter1\t$ratio2\t$parameter2\t$Expected\t$Observed\t$runstest\t$alignmentlengthORIGINAL\t$alignmentlengthTRIMMED\t$alignmentlengthNOINDELS\t[string length $sequence1]\t[string length $sequence2]\t$numberindels1\t$numberindels2\t$numberindels\t$COUNTalignednucleotides\t$COUNTconserved\t$COUNTdiverged\t$COUNTswitches\t$percentagedivergence"
	
		# Unset sequence and alignment variables
		unset alignment1 alignment2
		unset sequence1 sequence2
		unset annotation1 annotation2
		# Unset parameter variables
		unset parameter1 parameter2 
		unset ratio1 ratio2
		unset numberindels1 numberindels2 numberindels	

		# Unset runs test variables
		unset runstest
		unset COUNTalignednucleotides COUNTconserved COUNTdiverged COUNTswitches
		unset percentagedivergence Expected Observed
		# Unset length variables
		unset alignmentlengthORIGINAL alignmentlengthTRIMMED alignmentlengthNOINDELS
		
		# Unset variables
		unset ortholognumber intronnumber chromosome
		unset all finaldataset
	}
	unset alignmentfile alignmentfilelist

	# Unset variables	
	unset introncount numberintrons
	
    # Return to Intron_Data directory from ALIGNMENTS directory
	cd ..

	# Close output details file
	close $outDetails
	unset outDetails
	
	# Return from Intron_Data directory to main directory
   	cd ..

    puts "\nVariables open in INTRONS09_alignmentparameters : [info local]"

return
}
