##################################################################
#
# K.pl
#
# K with multiple-hit correction according to the models of 
#	Jukes-Cantor, 
#	Kimura (2-parameter), 
#	Tamura-Nei,
#	and Tamura-Kimura
# from individual alignment files.
# Models: see Nei & Kumar: Molecular Evolution and Phylogenetics, Oxford, p. 37-40.
#
# Input format:
# PHYLIP-style (*.nuc) alignments. We assume the following format:
#	-number of sequences (always 2) and sequence length on first line
# 	-first sequence name
#	-first sequence (spaces and newlines don't matter)
# 	-second sequence name
#	-second sequence (spaces and newlines don't matter)
#
# Version history:
# 31/12/0 Martin Lercher (TCL), using Liz's Kimura version as a template.
# 2/2/1		corrected formula for Kimura, 
#		added Jukes-Cantor, Tamura, HKY, Tamura-Nei models.
# 25/6/1	adapted to Perl and added Li's 1993 protocols
# 3/7/1		corrected CAA, CAG degeneracies
# 23/3/2	changed reading of input alignment -> allows proper PHYLIP format now
# 11/3/3	implemented the 'new' version by Tamura & Kumar (MBE 19:1727)
# 16/3/3	ignore codon structure (no Li)
#
# The models (Jukes-Cantor, Kimura, and Tamura-Nei models) were tested 
# against MEGA2 for 5 sequence pairs and gave correct results
#
# tested K4_TN against PAML for 5000+ Hs-Mm orthologs.
# 
##################################################################
use strict ;

	my $outFile = "K_noncoding_out.tab" ;
	my $minLength = 10 ;

#	$ARGV[0] || die "usage:	K_nonCoding.pl globPattern
#	files matched by globPattern must be alignments in PHYLIP (*.nuc) format.
#	[for more info see leading comment in source code]\n" ;

#	my @files = glob ($ARGV[0]) ;
	
	my @files = <*.nuc> ;
	printf "analysing %d files\n", scalar @files ;

	my %valid = ('A' => 1, 'T' => 1, 'G' => 1, 'C' => 1) ;

	open(OUT, ">>$outFile") || die "can't open $outFile" ;
	print OUT "file	gene1	gene2	N	K_JC	K_Kimura	K_TN	K_TK	raw_subst	raw_subst_per_site	GC\n" ;
	my $count = 0 ;
	FILE:
	foreach my $file (@files) {

		my ($name1, $seq1, $name2, $seq2) = readPhylipAln ($file) ;

		# loop over nucleotides
		my @nucs1 = split //, $seq1 ;
		my @nucs2 = split //, $seq2 ;
		unless (@nucs1 == @nucs2) {
			warn "$file: wrong sequence lengths" ;
			next FILE ;
		}
		my %n1 = my %n2 = my %substitutions = () ;
		while (@nucs1) {
			my $nuc1 = shift @nucs1 ;
			my $nuc2 = shift @nucs2 ;
			next unless ($valid{$nuc1} && $valid{$nuc2}) ;
			$n1{$nuc1}++ ;
			$n2{$nuc2}++ ;
			# count differences:
			if ($nuc1 ne $nuc2) {
				# transition (C<->T or A<->G) or transversion?:
				my $nucPair = "$nuc1$nuc2" ;
				if (($nucPair eq "CT") || ($nucPair eq "TC")) {
					$substitutions{transitionCT}++ ;
				} elsif (($nucPair eq "AG") || ($nucPair eq "GA")) {
					$substitutions{transitionAG}++ ;
				} else {
					$substitutions{transversion}++ ;
				}
			} 
		} # next nucs

		my $n2 = 0 ;
		my %n = () ;
		foreach my $x (qw/A T C G/) {
			$n{$x} = $n1{$x} + $n2{$x} ;
			$n2 += $n{$x} ;
		}
		my $n = $n2 / 2 ;
		my $subst = $substitutions{transitionCT} + $substitutions{transitionAG} + $substitutions{transversion} ;
		my $K_JC = my $K_Kimura = my $K_TN = my $K_TK = my $GC = my $K_noCorr = -1 ;
		if ($n) {
			$GC = ($n{G} + $n{C}) / $n2 ;
			$K_noCorr = $subst / $n ;

			# For Tamura-Nei, we have two separate transition frequencies (P = P_R + P_Y):
			my $P_R = $substitutions{transitionAG} / $n ;
			my $P_Y = $substitutions{transitionCT} / $n ;
			my $P = $P_R + $P_Y ;
			my $Q = $substitutions{transversion} / $n ;
			my $Y = ($n{T} + $n{C}) / $n2 ;
			my $R = ($n{G} + $n{A}) / $n2 ;
			my $A_G = $n{A} * $n{G} / ($n2 * $n2) ;
			my $T_C = $n{T} * $n{C} / ($n2 * $n2) ;
			# distinguish between sequences for Tamura-Kumar:
			my $f_AG = ($n1{A} * $n2{G} + $n1{G} * $n2{A}) / ($n * $n) ; 
			my $f_TC = ($n1{T} * $n2{C} + $n1{C} * $n2{T}) / ($n * $n) ; 
			my $f_RY = (($n1{A} + $n1{G}) * ($n2{T} + $n2{C}) 
				   + ($n1{T} + $n1{C}) * ($n2{A} + $n2{G})) / ($n * $n) ;

			# Tamura-Kumar model for heterogeneous substitution patterns (MBE 19:1727)
			if ($Y>0 && $R>0 && $f_AG>0 && $f_TC>0 && $f_RY>0) {
				my $aR = 1 - $R / $f_AG * $P_R - $Q / (2.* $R) ;
				my $aY = 1 - $Y / $f_TC * $P_Y - $Q / (2.* $Y) ;
				my $b  = 1 - $Q / $f_RY ;
				if ($aR>0 && $aY>0 && $b>0) {
					$K_TK =	- 2.* $A_G / $R * log($aR)
							- 2.* $T_C / $Y * log($aY)
							- 2.* ($R * $Y - $A_G * $Y / $R - $T_C * $R / $Y) * log($b) ;
				}
			}

			# Tamura-Nei model:
			if ($Y>0 && $R>0 && $A_G>0 && $T_C>0) {
				my $aR = 1 - $R / (2.* $A_G) * $P_R - $Q / (2.* $R) ;
				my $aY = 1 - $Y / (2.* $T_C) * $P_Y - $Q / (2.* $Y) ;
				my $b  = 1 - $Q / (2.* $R * $Y) ;
				if ($aR>0 && $aY>0 && $b>0) {
					$K_TN =	- 2.* $A_G / $R * log($aR) 
							- 2.* $T_C / $Y * log($aY)
							- 2.* ($R * $Y - $A_G * $Y / $R - $T_C * $R / $Y) * log($b) ;
				}
			}	

			# Jukes-Cantor model:
			my $a = 1.- 1.3333333 * ($P + $Q) ;
			$K_JC = - 0.75 * log($a) if ($a > 0);	

			# Kimura's model:
			$a = 1.- 2.* $P - $Q ;
			my $b = 1.- 2.* $Q ;
			$K_Kimura = - 0.5 * log($a) - 0.25 * log($b) if (($a > 0) && ($b > 0));
		}
		$file =~ s/.*\\// ;
		printf OUT "%s	%s	%s	%d	%.4f	%.4f	%.4f	%.4f	%d	%.4f	%.4f\n", 
			$file,$name1,$name2,$n,	$K_JC,	$K_Kimura,$K_TN,$K_TK,$subst,$K_noCorr, $GC ;
		printf     "\r%5d", $count++ ;
	} # next file

################################################################

sub readPhylipAln {
	my $file = shift ;

	unless (open(IN, $file)) {
		print "can't open $file\n" ;
		next FILE ;
	}
	my ($noSequences, $length) = split (" ", <IN>) ;
	while (<IN>) {
		chomp ;
		last if $_ ;
	}
	my $name1 = $_ ;
	my $seq1 = '' ;
	while (<IN>) {
		$_ =~ s/\s//g ;
		$seq1 .= $_ ;
		last if (length ($seq1) >= $length) ;
	}
	while (<IN>) {
		chomp ;
		last if $_ ;
	}
	my $name2 = $_ ;
	my $seq2 = '' ;
	while (<IN>) {
		$_ =~ s/\s//g ;
		$seq2 .= $_ ;
		last if (length ($seq2) >= $length) ;
	}
	close IN ;

	# convert to uppercase:
	$seq1 = uc $seq1 ;
	$seq2 = uc $seq2 ;
	
	return ($name1, $seq1, $name2, $seq2) ;
}